<?php
class Braintree_Donations {
	/**
	 * Contruct that configures the Braintree instance
	 */
	
	private $environment;
	
	private $merchantId;
	
	private $public_key;
	
	private $private_key;
	
	private $config;
	
	private $merchantAccountId;
	
	private $donation_form;
	
	private $paypal_enabled;
	
	public function __construct(){
		$this->load_configuration();
	}
	public function get_config(){
		return $this->config;
	}
	private function load_configuration(){
		
		if (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https')
			$_SERVER['HTTPS'] = 'on';
		
		$license_status = get_braintree_payments_config('braintree_payments_license_status');
		$this->config['api_keys'] = maybe_unserialize(get_braintree_payments_config('braintree_api_keys_config'));
		$this->config['messages'] = maybe_unserialize(get_braintree_payments_config('braintree_messages_config'));
		$this->config['merchant_accounts'] = maybe_unserialize(get_braintree_payments_config('braintree_merchant_account_id_config'));
		$this->config['donation_form_settings'] = get_braintree_payments_config('braintree_donation_form_settings');
		
		$production = ($this->config['api_keys']['environment'] === 'production') ? true : false;
		
		if($license_status == 'inactive'){
			check_braintree_donations_license();
			$license_status = get_braintree_payments_config('braintree_payments_license_status');
			$production = ($license_status == 'inactive') ? false : true;
		}
		
		if($production && !is_ssl()){
			$production = false;
		}
		
		$this->environment = ($production) ? 'production' : 'sandbox';
		$this->merchantId = ($production) ? $this->config['api_keys']['production']['merchantId'] : $this->config['api_keys']['sandbox']['merchantId'];
		$this->public_key = ($production) ? $this->config['api_keys']['production']['public_key'] : $this->config['api_keys']['sandbox']['public_key'];
		$this->private_key  = ($production) ? $this->config['api_keys']['production']['private_key'] : $this->config['api_keys']['sandbox']['private_key'];
		$this->merchantAccountId = ($production) 
		         ? $this->config['merchant_accounts']['production']['merchant_account_id'] :
		         $this->config['merchant_accounts']['sandbox']['merchant_account_id'];
		$this->donation_form = ($this->config['donation_form_settings']['donation_form'] == 'custom') ? 'custom' : 'dropin';
		$this->paypal_enabled = ($this->config['donation_form_settings']['paypal_active'] == 'true') ? true : false;
		
		if(class_exists('Braintree_Configuration')){
		Braintree_Configuration::environment($this->environment);
		Braintree_Configuration::merchantId($this->merchantId);
		Braintree_Configuration::publicKey($this->public_key);
		Braintree_Configuration::privateKey($this->private_key);
		}
	}
	
	public function process_payment(){
		return $this->execute_payment();
	}
	
	private function execute_payment(){
		$amount = $_POST['donation_amount'];
		$billing_name = $_POST['billing_name'];
		$nonce = ($this->environment == 'production') ? $_POST['nonce'] : 'fake-valid-nonce';
		
		if($_POST['payment_type'] == 'CreditCard'){
			$cardType = $_POST['payment_type'];
			$cardId = $_POST['cardId'];
			
			$args = array('amount'=>$amount,
						  'merchantAccountId'=>$this->merchantAccountId,
						  'paymentMethodNonce'=>$nonce,
						  'options'=>array(
							'submitForSettlement'=>true
			));
			
		}
		
		else if($_POST['payment_type'] == 'PayPalAccount'){
			
			$description = $this->config['messages']['receipt_message'];
			
			$args = array('amount'=>$amount,
						  'merchantAccountId'=>$this->merchantAccountId, 
						  'paymentMethodNonce'=>$nonce,
						  'options'=>array(
							'submitForSettlement'=>true, 'paypal'=>array('description'=>$description))
						  
			);
		}
		try{
			return  Braintree_Transaction::sale($args);
		}
		catch(Exception $e){
			return $e;
		}
		
	}
	
	public function generate_button_html(){
		$hmtl  = '<div class="braintreeDontaation">
					<input type="submit" class="donation-button" id="dontation_button"/>
				</div>';
		
		return $html;
	}
	
	/**
	 * Method that returns the client token generated by using the merchant account keys
	 * @return Ambigous <multitype:, mixed>
	 */
	private function get_client_token($use_customer = true){
		if($user_customer && $user = wp_get_current_user()){
			return Braintree_ClientToken::generate(array('customerId'=>$user->ID));
		}
		try{
		return Braintree_ClientToken::generate();
		}
		catch(Exception $e){
			return false;
		}
	}
	
	public function get_donation_form(){
		
		global $post;
		$url = ($post) ? get_permalink($post->ID) : $_SERVER['PHP_SELF'];
		$active_environment = $this->get_active_environment();
		
		if($this->donation_form == 'custom'){
			if($this->paypal_enabled){
				$paypal = $this->get_paypal_container();
			}
			
		$html = '<div class="braintree-donations">
				<div class="close-donation"><img src="'.BRAINTREE_PAYMENTS_ASSETS.'images/close-donation.png"/></div>
				'.$active_environment.'
				<form id="braintree_donations_form" class="donation-form" method="post" action="'.$url.'">
					<input type="hidden" id="client_token" value="'.$this->get_client_token().'"/>		
					
					<div class="payment-info-container">
						<div class="payment-info"><span>'.__('Cardholder Name', 'Braintree_Donations').'</span></div>
						<div class="payment-input"><input type="text" data-braintree-name="cardholder_name" id="billing_name" value""/></div>
					</div>
					<div class="payment-info-container">
						<div class="payment-info"><span>'.__('Credit Card Number', 'Braintree_Donations').'</span></div>
						<div class="payment-input"><input type="text" data-braintree-name="number" id="num_num" value=""/></div>
					</div>
				    <div class="payment-info-container-exp">
						<div class="payment-info"><span>'.__('Exp Date (MM/YYYY)', 'Braintree_Donations').'</span></div>
						<div class="payment-input"><input type="text" data-braintree-name="expiration_date" id="exp" value""/></div>
					</div>
					<div class="payment-info-container-cvv">
						<div class="payment-info"><span>'.__('CVV', 'Braintree_Donations').'</span></div>
						<div class="payment-input"><input type="password" data-braintree-name="cvv" id="cvv" value""/></div>
					</div>
						<div class="payment-info-container-donation">
						<div class="payment-info"><span>'.__('Donation Amount', 'Braintree_Donations').'</span></div>
							<div class="payment-input-donation"><div><span>'.$this->get_donation_currency(false).'</span></div><input type="text" id="donation_amount" value=""/></div>
							
						</div>
						'.$paypal.'
									
				    <div><button class="donation-form-button">'.__('Donate', 'Braintree_Donations') .'</button></div>
				</form>
						</div>
				    		<div id="donation-message"></div>
		<div class="donations-overlay"></div>
		<input type="hidden" id="ajax_url" value="'.admin_url().'admin-ajax.php' .'"/>';
		
		}
		else if($this->donation_form == 'dropin'){
			
			$html = '<div id="dropin-container" class="braintree-donations">
						<div class="close-donation"><img src="'.BRAINTREE_PAYMENTS_ASSETS.'images/close-donation.png"/></div>
						'.$active_environment.'
								<form id="braintree_donations_form" class="donation-form" method="post" action="'.$url.'">
							<input type="hidden" id="client_token" value="'.$this->get_client_token(false).'"/>
						<div class="payment-info-container">
							<div class="payment-info"><span>'.__('Donation Amount', 'Braintree_Donations').'</span></div>
							<div class="payment-input-donation"><span>'.$this->get_donation_currency().'</span><input type="text" id="donation_amount" value""/></div>
						</div>
						<div><button class="donation-form-button donation-form-button-dropin">'.__('Donate', 'Braintree_Donations') .'</button></div>
						</form>
					</div>
			<div id="donation-message"></div>
			<div class="donations-overlay"></div>
			<input type="hidden" id="ajax_url" value="'.admin_url().'admin-ajax.php' .'"/>';
		}
		return $html;
	}
	
	public function get_donation_button(){
		$html = '<div class="braintree-donations-button">
						<button type="button" id="donate-button" 
				style="background-color: '.$this->config['donation_form_settings']['donation_button']['background_color'].'; border-color: '.$this->config['donation_form_settings']['donation_button']['border_color'].'">'
						.__('Donate', 'Braintree_Donations').'</button>
				 </div>';
		
		return $html;
			
	}
	
	private function get_donation_currency(){
		$current_currency = $this->config['api_keys'][$this->environment]['currency'];
		$currencies = Currencies::get_currencies();
		foreach($currencies as $currency => $symbol){
			if($current_currency == $currency){
				return $symbol;
			}
		}
	}
	
	public static function test_braintree_connection($args = ''){
		Braintree_Configuration::merchantId($args['merchantId']);
		Braintree_Configuration::environment($args['environment']);
		Braintree_Configuration::publicKey($args['publicKey']);
		Braintree_Configuration::privateKey($args['privateKey']);
		
		try{
		$response = Braintree_Customer::find('sags2352kj5h2jjgjk24bhjkf');
			if($response->success){
				$response['message'] = 'Connection to Braintree was successful';
				$response['code'] = 'success';
				return $response;
			}
		} 
		catch(Exception $e){
			if(is_a($e, 'Braintree_Exception_Authentication')){
				$response['message'] = 'Uh oh, your API keys are not correct. Please verify your entries';
				$response['code'] = 'error';
			}
			if(is_a($e, 'Braintree_Exception_Authorization')){
				$response['message'] = 'Uh oh, you are not authorized to perform this function. Please contact Braintree';
				$response['code'] = 'error';
			}
			if(is_a($e, 'Braintree_Exception_NotFound')){
				$response['message'] = 'Connection test to Braintree was successful. Let\'s make some dough!';
				$response['code'] = 'success';
			}
			if(is_a($e, 'Braintree_Exception_UpgradeRequired')){
				$response['message'] = 'Connection test to Braintree was successful. Let\'s make some dough!';
				$response['code'] = 'success';
			}
			if(is_a($e, 'Braintree_Exception_ServerError')){
				$response['message'] = 'Something has gone wrong on Braintree\'s server. Please test 
						again later';
				$response['code'] = 'error';
			}
			if(is_a($e, 'Braintree_Exception_DownForMaintenance')){
				$response['message'] = 'Braintree\'s server\'s are down. Please try again later';
				$response['code'] = 'error';
			}
			if(is_a($e, 'Braintree_Exception_Unexpected')){
				$response['message'] = 'Houston, we have a problem. An unexptected error has occured. Please try the connection again';
				$response['code'] = 'error';
			}
			
			return $response;
		}
		
	}
	
	function get_paypal_container(){
		return '<div class="payment-info-container-paypal">
				<div class="paypal-container">
				<div><span>Donate With Paypal</span></div>
					<div id="paypal-container"></div>
				<div class="cancel-paypal"><span id="cancel-paypal">Cancel Paypal</span></div>
				</div>
				</div>
				<input type="hidden" id="paypal_active"/>';
	}
	private function get_active_environment(){
		if($this->environment == 'sandbox'){
			return '<div class="active-environment-message"><span>Sandbox Active</span></div>';
		}
	}
}
?>