<?php

function initialize_braintree_subscription_functions(){
	Braintree_WooSubscription::init();
}
add_action('init', 'initialize_braintree_subscription_functions');

class Braintree_WooSubscription {
	
	public static $config;
	public static $environment;
	private static $public_key;
	private static $private_key;
	private static $merchantId;
	public static $merchantAccountId;
	public static $log;
	private static $init_called;
	private static $id = 'braintree_payment_gateway';
	
	public function __construct(){
	
	}
	
	public static function init(){
		if(self::$init_called){
			return;
		}
		$license_status = get_braintree_payments_config('braintree_payments_license_status');
		self::$config['api_keys'] = maybe_unserialize(get_braintree_payments_config('braintree_api_keys_config'));
		self::$config['merchant_accounts'] = maybe_unserialize(get_braintree_payments_config('braintree_merchant_account_id_config'));
		self::$config['woopayments'] = maybe_unserialize(get_braintree_payments_config('braintree_payments_woocommerce_config'));
		self::$config['subscription_config'] = get_braintree_payments_config('braintree_subscription_config');
			
		$production = (self::$config['api_keys']['environment'] == 'production') ? true : false;
			
		if($license_status == 'inactive' && $production){
			check_braintree_donations_license();
			$license_status = get_braintree_payments_config('braintree_payments_license_status');
			$production = ($license_status == 'inactive') ? false : true;
		}
			
		self::$environment = ($production) ? 'production' : 'sandbox';
		self::$merchantId = self::$config['api_keys'][self::$environment]['merchantId'];
		self::$public_key = self::$config['api_keys'][self::$environment]['public_key'];
		self::$private_key  = self::$config['api_keys'][self::$environment]['private_key'];
		
		self::$merchantAccountId = self::$config['merchant_accounts'][self::$environment]['merchant_account_id'];
		
		Braintree_Configuration::environment(self::$environment);
		Braintree_Configuration::merchantId(self::$merchantId);
		Braintree_Configuration::publicKey(self::$public_key);
		Braintree_Configuration::privateKey(self::$private_key);
		
		self::$log = new WC_Logger();
			
		//Adds the static method to the cancel subscription hook. 
		add_action('cancelled_subscription_braintree_payment_gateway', __CLASS__.'::cancel_braintree_subscription', 10, 2);
		
		//Adds the static method to the update order meta hook which is called from the admin order page. 
		add_action('woocommerce_process_shop_order_meta', __CLASS__.'::update_subscription_recurring_amount', 10, 2);
		
		//Adds the static method to the subscription update payment method hook. 
		add_action('woocommerce_subscriptions_updated_recurring_payment_method_to_'.self::$id, __CLASS__.'::change_braintree_subscription_paymentMethod', 10, 4);
		self::$init_called = true;
	}
	
	public static function process_new_subscription($order, Braintree_For_Woocommerce $gateway){
		//check if this is a payment method change. If so, exit method without executing code. 
		if(WC_Subscriptions_Change_Payment_Gateway::$is_request_to_change_payment){
			//If code made it this far, then payment method was changed successfully. 
			return array('result'=>'success', 
					'redirect'=>self::get_paymentMethod_change_redirectUrl()
			);
		}
		$use_token = (isset($_POST['use_saved_card'])) ? true : false;
		
		//$nonce = ($gateway->environment == 'production') ? $_POST['payment_method_nonce'] : 'fake-valid-nonce';
		$nonce = (isset($_POST['payment_method_nonce'])) ? $_POST['payment_method_nonce'] : false;
		
		//Set the sunscription id equal to the order id for easy reference.
		$args = array('merchantAccountId'=>$gateway->merchantAccountId,
				'id'=>$order->id
		);
		$user_id = wp_get_current_user()->ID;
		$braintree_id = get_user_meta($user_id, 'braintree_vault_id', true);
		
		if($user_id && !$use_token){
			
			if(empty($braintree_id) || !$braintree_id && $nonce){
				try{
				$result = Braintree_Customer::create(array(
						'paymentMethodNonce'=>$nonce));
						
					if($result->success){
						$braintree_id = $result->customer->id;
						update_user_meta($user_id, 'braintree_vault_id', $braintree_id);
						$args['customerId'] = $braintree_id;
						$args['paymentMethodToken'] = $result->customer->paymentMethods[0]->token;
					}
					elseif(! $result->success){
						wc_add_notice(__('Your subsciption could not be created at this time.', 'Braintree_Woocommerce'), 'error');
						return array('result'=>'failure',
								'redirect'=>''
						);
					}		
				
				} catch(Exception $e){
					wc_add_notice(__('Your subsciption could not be created at this time.', 'Braintree_Woocommerce'), 'error');
					return array('result'=>'failure', 
							'redirect'=>''
					);
				}
				
			}
			else {
				try {
					$result = Braintree_PaymentMethod::create(array(
							'paymentMethodNonce'=>$nonce,
							'customerId'=>$braintree_id,
							'options'=>array('failOnDuplicatePaymentMethod'=>true)
					));
					if($result->success){
						$args['paymentMethodToken'] = $result->paymentMethod->token;
					}
					elseif(! $result->success){
						//check if the nonce represents a duplicate card. If so, the nonce can be used in a subscription.
						foreach($result->errors->deepAll() as $error){
							if($error->code == '81724'){
								$args['paymentMethodNonce'] = $nonce;
							}
						}
					}
				}
				catch (Exception $e){
					wc_add_notice(__('Your payment method could not be charged at this time.
							 Please try again', 'Braintree_Woocommerce'), 'error');
					return array('result'=>'failure',
							'redirect'=>''
					);
				}
					
			}
		}
		else {
			//fetch the payment token from using the array index;
			$token = ($use_token) ? $_POST['existing_card'] : null;
			try {
				$customer = Braintree_Customer::find($braintree_id);
			
				$token = $customer->paymentMethods[$token]->token;
			
				$args['paymentMethodToken'] = $token;
			
			} 
			catch(Braintree_Exception_NotFound $e){
					wc_add_notice(__('Your customer Id could not be verified. Please contact the site owner.', 'Braintree_Woocomerce'), 'error');
					return array('result'=>'failure',
							'redirect'=>'');
			}
			
		}
		
		$plan_id = self::get_braintree_subscription_planId($order);
		
		if($plan_id == '1001'){
			wc_add_notice(__('You cannot add more that one subscription to your cart at a time.
					 Please purchaes subscriptions one at a time', 'Braintree_Woocommerce'), 'error');
			return array('result'=>'failure',
					'redirect'=>'');
		}
		elseif($plan_id == '1002'){
			wc_add_notice(__('This subscription plan has not been configured. Please contact the site owner.'), 'error');
			return array('result'=>'failure', 
					'redirect'=>''
			);
		}
		
		//$plan_id was found so add it to the subscription. 
		$args['planId'] = $plan_id;
		
		$price_per_period = WC_Subscriptions_Order::get_price_per_period($order);
		$args['price'] = $price_per_period;
		//returns the number of units that the subscription lasts
		$subsc_length = WC_Subscriptions_Order::get_subscription_length($order);
		$subsc_unit = WC_Subscriptions_Order::get_subscription_period($order);
		
		if($subsc_length){
			//numberofBillingCycles must be in months because Braintree only supports months for their billing cycles. 
			switch($subsc_unit){
				case 'week':
					$subsc_unit = 'month';
					$subsc_length = round(($subsc_length / 52) * 12); //convert weeks to months and round to the nearest month. 
					break;

				case 'day':
					$subsc_unit = 'month';
					$subsc_length = round(($subsc_length / 365) * 12); //convert days to months and round to the nearest month.
					break;

				case 'year':
					$subsc_unit = 'month';
					$subsc_length = ($subsc_length * 12 ); //convert year(s) to months.
					break;
			}
			$args['numberOfBillingCycles'] = $subsc_length;
		}
		
		//$signup_fee = WC_Subscriptions_Order::get_sign_up_fee($order);
		
		//Returns the number of periods that the trial lasts.*/
		$trial_length = WC_Subscriptions_Order::get_subscription_trial_length($order);
		
		if($trial_length){
			
			$trial_unit = WC_Subscriptions_Order::get_subscription_trial_period($order);
			
			switch($trial_unit){
				case 'week':
					$trial_unit = 'day';
					$trial_length = $trial_length * 7; //convert the trial length to days
					break;
					
				case 'year':
					$trial_unit = 'month';
					$trial_length = $trial_length * 12; //convert year(s) to months;
						
			}
			$args['trialPeriod'] = true;
			$args['trialDuration'] = $trial_length;
			$args['trialDurationUnit'] = $trial_unit;
		}
		
		try {
			$result = Braintree_Subscription::create($args);
			if($result->success){
				global $woocommerce;
				if($gateway->order_status == 'wc-completed'){
					$order->payment_complete($result->subscription->transactions[0]->id);
				}
				else{
					$order->update_status($gateway->order_status, __('Credit card payment processed', 'woocommerce'));
					update_post_meta($order-id, '_transaction_id', $result->subscription->transactions[0]->id);
				}
				$woocommerce->cart->empty_cart();
				if($user_id && !$braintree_id){
					$new_id = $result->transaction->customerDetails->id;
					update_user_meta($user_id, 'braintree_vault_id', $new_id);
				}
				return array(
						'result'=>'success',
						'redirect'=>WC_Payment_Gateway::get_return_url($order));
			}
			if(! $result->success){
				if($result->errors){
					foreach($result->errors->deepAll() as $error)
						wc_add_notice($error->message, 'error');
				}
				else {
					wc_add_notice(__('Your payment method could not be verified.', 'Braintree_Woocommerce'), 'error');
				}
			}
		}
		catch (Exception $e){
		
		}
	}
	
	private static function create_braintree_user($user_id){
		
	}
	private static function get_braintree_subscription_planId(WC_Order $order){
		if(count($order->get_items()) > 1){
			return '1001';
		}
		foreach($order->get_items() as $item){
			$product_id = $item['product_id'];
			break;
		}
		
		if($planId = get_post_meta($product_id, 'braintree_subscription_id', true)){
			return $planId;
		}
		
		else {
			return '1002';
		}
	}
	/**
	 * Takes the order and product id as parameters. Uses the order id as the subscription id and calls the Braintree API
	 * to cancel the subscription.
	 * @param WC_Order $order
	 * @param unknown $product_id
	 */
	public static function cancel_braintree_subscription(WC_Order $order, $product_id){
		//Braintree config has been initialized in the init() method.
		//Check if this is a payment method update. If so, return without executing code. 
		if(WC_Subscriptions_Change_Payment_Gateway::$is_request_to_change_payment){
			return;
		}
		
		try{
			$result = Braintree_Subscription::cancel($order->id);
			if($result->success){
				wc_add_notice(__('Subscription '.$order->id.' has been cancelled.', 'Braintree_Woocommerce'), 'success');
				return;
			}
			else {
				wc_add_notice(__('If you are seeing this message it means you have cancelled a 
						subscription that was pending.'), 'error');
			}
		}
		catch(Exception $e){
			if(is_admin(wp_get_current_user())) {
				wc_add_notice(__('There was an error cancelling the subscription. If the issue persists, please manually cancel
						the subscription by logging in to your braintree account.', 'Braintree_Woocommerce'), 'error');
			}
			else {
				wc_add_notice(__('There was an error cancelling the subscription. If the issue persists, please contact
						your subscription provider.'), 'error');
			}
		}
		
	}
	
	public static function update_subscription_recurring_amount($post_id, $post){
		if(! $post_id || !isset($_POST['_order_recurring_total'] )){
			return;
		}
		
		$order = new WC_Order($post_id);
		$new_amount = get_post_meta($post_id, '_order_recurring_total', true);
		
		if(! self::$init_called){
			self::$init();
		}
		
		try{
			$subscription = Braintree_Subscription::find($order->id);
				if($subscription instanceof Braintree_Subscription){
					$subsc_amount = $subscription->price;
				}
				else {
					self::$log->add('braintree', 'Subscription '.$order->id.' could not be found. Check for inconsistancies in Braintree
							or your wordpress database.');
					return;
				}
			
		} catch(Exception $e){
			self::$log->add('braintree', 'There was an error while searching for 
					the subscription '.$order->id.' in your braintree vault.');
			return;
		}
		
		if($new_amount == $subsc_amount){
			return; //the amounts match between wordpress and braintree therefore no change was made to the recurring amount. 
		}
		$attrs = array('id' => $order->id, 'price' => $new_amount);
		try{
			$result = Braintree_Subscription::update($order->id, $attrs);
				if($result->success){
					self::$log->add('braintree', 'Subscription '.$order->id.' was successfully updated');
					return;
				}
				else {
					self::$log->add('braintree', 'Subscription '.$order->id.' could not be updated at this time');
					return;
				}
		} catch(Exception $e){
			self::$log->add('braintree', 'There was an error while updating the recurring amount
					for subscription '.$order->id.' in your braintree vault.');
			return;
		}
	}
	
	public static function change_braintree_subscription_paymentMethod(WC_Order $order, $subscription_key, $old_payment_method){
		//change the payment method for the subscription
		$attrs = array();
		$use_token = (isset($_POST['use_saved_card'])) ? true : false;
		
		//$nonce = ($gateway->environment == 'production') ? $_POST['payment_method_nonce'] : 'fake-valid-nonce';
		$nonce = (isset($_POST['payment_method_nonce'])) ? $_POST['payment_method_nonce'] : false;
		
		//Set the merchantAccountId for this subscription.
		$attrs['merchantAccountId'] = self::$merchantAccountId;
		
		$user_id = wp_get_current_user()->ID;
		$braintree_id = get_user_meta($user_id, 'braintree_vault_id', true);
		
		//If customer isn't using a saved payment method, need to add the new payment method to the vault
		if(!$use_token){
		try {
					$result = Braintree_PaymentMethod::create(array(
							'paymentMethodNonce'=>$nonce,
							'customerId'=>$braintree_id,
							'options'=>array('failOnDuplicatePaymentMethod'=>true)
					));
					if($result->success){
						$attrs['paymentMethodToken'] = $result->paymentMethod->token;
					}
					elseif(! $result->success){
						//check if the nonce represents a duplicate card. If so, the nonce can be used in a subscription.
						foreach($result->errors->deepAll() as $error){
							if($error->code == '81724'){
								WC_Subscriptions::add_notice(__('You have tried to enter a payment method which already exists.', 'Braintree_Woocommerce'), 'error');
							}
							else {
								WC_Subscriptions::add_notice($error->message);
							}
						}
						return;
					}
				}
				catch(Exeption $e){
					wc_add_notice(__('Your payment method could not be charged at this time.
							 Please try again', 'Braintree_Woocommerce'), 'error');
						return;
				}
		}
		//get the payment method token and use it for the new subscription payment method.
		else {
			$token_index = $_POST['existing_card'];
			$customer = Braintree_Customer::find($braintree_id);
			$token = $customer->paymentMethods[$token_index]->token;
			$attrs['paymentMethodToken'] = $token;
		}
		try {
			$result = Braintree_Subscription::update($order->id, $attrs);
				if($result->success){
					WC_Subscriptions::add_notice(__('Your payment method for subscription '.$order->id.' was updated successfully', 'Braintree_Woocommerce'), 'success');
					return;
				}
				elseif($result->errors){
					foreach($result->errors->deepAll() as $error){
						WC_Subscriptions::add_notice($error->message, 'error');
					}
				}
		}
		catch(Exception $e){
			WC_Subscriptions::add_notice(__('Your payment method for subscription '.$order->id.' could not be changed at this time.', 'Braintree_Woocommerce'), 'error');
			return;
		}
	}
	
	public static function get_paymentMethod_change_redirectUrl(){
		return get_permalink( self::$config['subscription_config']['change_paymentmethod_redirect_page'] );
	}
	
}
?>